/*
 * Copyright 2006 Takahiro Nakamura.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package woolpack.el;

import java.util.HashMap;
import java.util.Map;

import woolpack.TestBean;
import junit.framework.TestCase;

public class PropertyELTest extends TestCase {

	public void testConstructor(){
		try{
			new PropertyEL(null, TypeConverter.NOP);
			fail();
		}catch(final NullPointerException expected){
		}
		try{
			new PropertyEL("", TypeConverter.NOP);
			fail();
		}catch(final StringIndexOutOfBoundsException expected){
		}
		try{
			new PropertyEL("key0", null);
			fail();
		}catch(final NullPointerException expected){
		}
	}
	
	public void testNormalBean(){
		final TestBean bean = new TestBean();
		
		assertEquals(new Integer(0), new PropertyEL("myInt", TypeConverter.NOP).getValue(bean));
		assertEquals("0", new PropertyEL("myInt", TypeConverter.NOP).getValue(bean, String.class));
		new PropertyEL("myInt", TypeConverter.NOP).setValue(bean, new Integer(2));
		new PropertyEL("myInt", TypeConverter.NOP).setValue(bean, new Integer(1));
		assertEquals(new Integer(1), new PropertyEL("myInt", TypeConverter.NOP).getValue(bean));

		try{
			new PropertyEL("yourInt", TypeConverter.NOP).getValue(bean);
			fail();
		}catch(final RuntimeException expected){
			assertEquals("property not found: class woolpack.TestBean: yourInt", expected.getMessage());
		}
		try{
			new PropertyEL("yourInt", TypeConverter.NOP).getValue(bean, String.class);
			fail();
		}catch(final RuntimeException expected){
			assertEquals("property not found: class woolpack.TestBean: yourInt", expected.getMessage());
		}
		try{
			new PropertyEL("yourInt", TypeConverter.NOP).setValue(bean, new Integer(2));
			fail();
		}catch(final RuntimeException expected){
			assertEquals("property not found: class woolpack.TestBean: yourInt", expected.getMessage());
		}
	}
	
	public void testNormalMap(){
		final Map<String, Object> map = new HashMap<String,Object>();
		map.put("param0", new Integer(3));
		assertEquals(new Integer(3), new PropertyEL("param0", TypeConverter.NOP).getValue(map));
		assertEquals(new Integer(3), new PropertyEL("param0", TypeConverter.NOP).getValue(map, String.class));
		
		new PropertyEL("param1", TypeConverter.NOP).setValue(map, "value2");
		new PropertyEL("param1", TypeConverter.NOP).setValue(map, "value1");
		assertEquals("value1", new PropertyEL("param1", TypeConverter.NOP).getValue(map));

		assertNull(new PropertyEL("param2", TypeConverter.NOP).getValue(map));

		new PropertyEL("param3", TypeConverter.NOP).setValue(map, null);
		assertNull(new PropertyEL("param3", TypeConverter.NOP).getValue(map));
	}
	
	public void testEquals(){
		assertEquals(new PropertyEL("key0").hashCode(), new PropertyEL("key0").hashCode());
		assertEquals(new PropertyEL("key0"), new PropertyEL("key0"));
		assertFalse(new PropertyEL("key0").equals(new PropertyEL("key1")));
		assertFalse(new PropertyEL("key0").equals(new Object()));
	}
}
