/*
 * Copyright 2006 Takahiro Nakamura.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package woolpack.validator;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;

import woolpack.TestUtils;
import woolpack.utils.MapBuilder;

import junit.framework.TestCase;

public class ValidValuesValidatorTest extends TestCase {

	public void testConstructor(){
		try{
			new ValidValuesValidator(null);
			fail();
		}catch(final NullPointerException expected){
		}
	}

	public void testInterpret(){
		final ValidatorExpression expression = new ValidValuesValidator(Arrays.asList("v0", "v2", "v4"));
		final ValidatorContext context = new ValidatorContext();
		context.setInputMap(new HashMap<String,List<Object>>());
		context.setTmpKey("key0");

		context.setTmpValue("v0");
		assertTrue(expression.interpret(context));
		assertTrue(TestUtils.equals(MapBuilder.get(new HashMap<String,Object>()).put("key0", Arrays.asList("v0")).get(), context.getInputMap()));
		
		context.setTmpValue("v1");
		assertFalse(expression.interpret(context));
		assertTrue(TestUtils.equals(MapBuilder.get(new HashMap<String,Object>()).put("key0", Arrays.asList("v1")).get(), context.getInputMap()));
	}
	
	public void testAppendTo() throws IOException{
		final ValidatorExpression expression = new ValidValuesValidator(Arrays.asList("v0", "v2", "v4"));
		final StringBuilder sb = new StringBuilder();
		expression.appendTo(sb);
		assertEquals("new ValidValuesValidator([\"v0\",\"v2\",\"v4\"])", sb.toString());
	}
	
	public void testAddMessageTo(){
		final ValidatorExpression expression = new ValidValuesValidator(Arrays.asList("v0", "v2", "v4"));
		final Collection<String> expectedCollection = new ArrayList<String>();
		expression.addMessageTo(expectedCollection);
		assertTrue(expectedCollection.isEmpty());
	}
}
